/************************************************************************/
/* Copyright 2004-2007 SYS Technologies, Inc.  All rights reserved.     */
/*               SYS Technologies, Inc. - Proprietary                   */
/* This software contains information of SYS Technologies and is not    */
/* to be disclosed or used except in accordance with applicable         */
/* agreements. Use of this DOPPLER VUE software is governed as          */
/* expressly authorized in the relevant agreement between               */
/* SYS Technologies and the customer, as well as copyright laws and     */
/* international treaties. Unauthorized distribution or duplication     */
/* will result in severe criminal penalties.                            */
/*               SYS Technologies, Inc. - Proprietary                   */
/************************************************************************/

/**************************************************************************************************
    GLOBAL MEMBERS
**************************************************************************************************/

//WMI polling interval
var _interval         //ms
var _intervalSettingID = -1;
var _forceWMITimeoutID = -1;

//WMI connection/authentication settings
var _local,_computer, _domain, _username, _password;

//initial measurements for cpu usage calculations
var _procTime1 = -1;
var _timestamp1 = -1;
var _procTimes1 = new Array();
var _timestamps1 = new Array();

//calculated values for cpu usage
var _procTime = -1;
var _procTimes = new Array();
var _procSpeed = -1;
var _procCount = 0;

//vertical slider state
var _open;

var _locator;               //WMI Scripting locator object
var _service;               //WMI Scripting connection object

var _processingWMI = false;
var _queryExecutionStart;
var _settingsWindowOpen = false;

/**************************************************************************************************
    logging
**************************************************************************************************/
var oFSO = new ActiveXObject("Scripting.FileSystemObject");
var bDebug = oFSO.FileExists(System.Gadget.path + "\\debug.txt");
try{
	if (bDebug)
		var debugLogFile = oFSO.OpenTextFile(System.Gadget.path + "\\debug.txt", 2);
} catch(err) {bDebug = false; debugLog("Open debug.txt error" + err.name + " - " + err.message)}

/*
    debugLog - debug log utility
*/
function debugLog(str) {
	try{
		System.Debug.outputString(str);
		if (bDebug)
			debugLogFile.WriteLine(str);
	} catch(err) {}
}
/**************************************************************************************************
    end logging
**************************************************************************************************/

/**************************************************************************************************
    END GLOBAL MEMBERS
**************************************************************************************************/


/**************************************************************************************************
    init
**************************************************************************************************/
function initGadget()
{
    //debugger;
    System.Gadget.background = "images/backgrounds/pro_1_back.png";
    System.Gadget.settingsUI = "settings.htm";
    System.Gadget.onSettingsClosed = procSettingsClosedEvent;
    System.Gadget.onShowSettings = procShowSettingsEvent;

    initGraphics();
    initSettings();
        
    //set hostname display values
    if(_computer == "")
        _local = true;
    if(_local == true)
    {
        hostname.title = "Polling local machine for WMI data." + "\n" + 
                        "Use settings page to configure a remote connection.";
        hostname.innerText = "localhost";
    }
    else
    {
        hostname.innerText = _computer;
        hostname.title = "Polling " + _computer + " for WMI data." + "\n" + 
                        "Use settings page to change the remote connection.";
    }
    
    //check the website for gadget updates (notification)
    //check for updates immediately
    setTimeout(checkForUpdates, 500);  //delay so the GUI has time to display
    window.setInterval(checkForUpdates, 600000); //600000 - every 10 minutes
    
    //execute WMI queries
    setTimeout(ExecuteQueries, 1000);
    _intervalSettingID = window.setInterval(ExecuteQueries, DEFAULT_POLLING_INTERVAL);   //5000 - every 5 seconds
}

function initGraphics()
{
    UpdatePositions(DEFAULT_PROCESSOR_COUNT, DEFAULT_SLIDER_STATE_OPEN);
    processorTimeInner.style.height = "0%";
    
    var i = 0;
    for (i; i<4; i++)
    {
        var procDiv = document.getElementById("proc" + i.toString() + "inner");
        procDiv.style.width = "0%";
    }
    
    percentMemoryUsedInner.style.height = "0%";

}

/**************************************************************************************************
    END init
**************************************************************************************************/




/**************************************************************************************************
    settings
**************************************************************************************************/

/*
    initSettings - initialize any default values for System.Gadget.settings items
*/
function initSettings()
{
    System.Gadget.settingsUI = "settings.htm";
    System.Gadget.onSettingsClosed = procSettingsClosedEvent;
    System.Gadget.onShowSettings = procShowSettingsEvent;
    
    //read saved settings values
    var interval = System.Gadget.Settings.read( "pollingInterval" );    
    if (interval.toString() == "")
        _interval = DEFAULT_POLLING_INTERVAL;
        
    var local = System.Gadget.Settings.read( "local" );
    if(local.toString() == "")
        local = DEFAULT_LOCAL_STATUS;
    
    _local = local;
    System.Gadget.Settings.write( "local", _local );
    
    _computer = System.Gadget.Settings.read( "computer" );
    _domain = System.Gadget.Settings.read( "domain" );
    _username = System.Gadget.Settings.read( "username" );
    _password = System.Gadget.Settings.read( "password" );
    
}
/*
	procShowSettingsEvent - process settings show event
*/
function procShowSettingsEvent()
{
    //close slider
    UpdatePositions(_procCount, false);
    _settingsWindowOpen = true;
}
/*
    procSettingsClosedEvent - process settings closed event
*/
function procSettingsClosedEvent(event)
{
	debugLog(new Date() + ":  procSettingsClosedEvent start.");
	// process settings closed event
	if (event.closeAction == event.Action.commit)
	{
		validateNewSettings();
	}
    _settingsWindowOpen = false;
	
	debugLog(new Date() + ":  procSettingsClosedEvent end.");
}
/*
    validateNewSettings
*/
function validateNewSettings()
{
    var methodName = "validateNewSettings";
    var resetWMI = false;
    var resetDisplay = false;
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        //polling interval
        var newInterval = Number( System.Gadget.Settings.read( "pollingInterval" ) );
        if(newInterval != _interval)
        {
            _interval = newInterval;
            resetWMI = true;
        }
        
        //WMI login information
        var newLocal = System.Gadget.Settings.read( "local" );
        var newComputer = System.Gadget.Settings.read( "computer" );
        var newDomain = System.Gadget.Settings.read( "domain" );
        var newUsername = System.Gadget.Settings.read( "username" );
        var newPassword = System.Gadget.Settings.read( "password" );
        
        if(_local != newLocal || _computer != newComputer)
        {
            resetDisplay = true;
            resetWMI = true;
        }
        else if(_domain != newDomain || _username != newUsername || _password != newPassword)
        {
            resetDisplay = true;
            resetWMI = true;
        }
        else
            resetDisplay = false;
            
        _local = newLocal;
        _computer = newComputer;
        _domain = newDomain;
        _username = newUsername;
        _password = newPassword;
        
        if(_local)
        {
            hostname.innerText = "localhost";
            hostname.title = "Polling local machine for WMI data." + "\n" + 
                            "Use settings page to configure a remote connection.";
        }
        else
        {
            hostname.innerText = _computer;
            hostname.title = "Polling " + _computer + " for WMI data." + "\n" + 
                            "Use settings page to change the remote connection.";
        }
        
        if(resetDisplay)
        {
            _procCount = 1;
            UpdatePositions(_procCount, false);
            ClearDisplay(); //clear all fields
        }
        
        if(resetWMI)
        {
            CleanUpWMI();
            
            if (Number(_intervalSettingID) > 0)
                clearTimeout(_intervalSettingID);
                
            setTimeout(ExecuteQueries, 500);
            _intervalSettingID = window.setInterval(ExecuteQueries, _interval);
        }
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}

/**************************************************************************************************
    END settings
**************************************************************************************************/




/**************************************************************************************************
    gadget updates
**************************************************************************************************/

/*
	checkForUpdates - check for gadget updates
*/
function checkForUpdates() { 
    if (_settingsWindowOpen == false)
    {
        //debugLog(new Date() + ":  --------------- checking for updates");
        if (window.XMLHttpRequest) { // Mozilla, Safari,... 
           http_request = new XMLHttpRequest();        
        } else if (window.ActiveXObject) { // IE 
           http_request = new ActiveXObject("Microsoft.XMLHTTP"); 
        } 
        
        //This property is an event handler which is triggered each time the state of the request changes. 
        //The states run from 0 (uninitialized) to 4 (complete). By having the function xmlhttpChange() 
        //check for the state changing, we can tell when the process is complete and continue only if it has been successful.
        http_request.onreadystatechange = onUpdatesArrived;
         
        var websiteText = "http://lighthouse.dopplervue.com/dvue_light_3.php";
        http_request.open('GET', websiteText, true); 
        http_request.setRequestHeader( "If-Modified-Since", "Sat, 1 Jan 2000 00:00:00 GMT" );
        http_request.send(null); 
    }
    else
    {
        debugLog(new Date() + ":  settings window is open ... skipping");
    }
}

/*
	onUpdatesArrived - check for gadget updates
*/
function onUpdatesArrived() 
{ 
   if (http_request.readyState == 4) { 
       if (http_request.status == 200) { 
            //4/12/2007 9:22:31 AM#0#This is my tooltip text.
            var text = http_request.responseText
            arr = text.split("#");
            if (arr.length == 2)
            {
                lastVersion = System.Gadget.Settings.read("lastUpdate");
                if (lastVersion.length == 0)
                    lastVersion = 0;
                currentVersion = arr[0];
                var now = new Date();
                //if the current version is greater than the last version and
                //the current version is less then todays date then turn the light on
                debugLog("Current Version = " + currentVersion);
                debugLog("Now = " + now.getTime());
                if ((Number(currentVersion) > Number(lastVersion)) && 
                    (Number(currentVersion) > now.getTime()))
                {
                    topLink.src="images/backgrounds/top_pro_on.png";
                    //set tooltip text
                    topLink.alt = arr[1];
                    System.Gadget.Settings.write("lastUpdate", currentVersion);
                }
            }
       }
   }
} 

/*
	resetUpdateNotification - function is called when the update link (titlebar) is clicked
*/
function resetUpdateNotification()
{
    topLink.src="images/backgrounds/top_pro_off.png";
    topLink.alt = "Click here to learn more about DopplerVUE."
}

/**************************************************************************************************
    END gadget updates
**************************************************************************************************/




/**************************************************************************************************
    WMI Query execution
**************************************************************************************************/

/*
	ExecuteQueries - begin executing WMI queries
*/
function ExecuteQueries()
{
    var methodName = "ExecuteQueries";
        
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        if(_settingsWindowOpen == false && _processingWMI == false)
        { 
            _processingWMI = true;
            _queryExecutionStart = new Date();
                
            var init = InitWMI();
            if(!init)
            {
                debugLog(new Date() + ":  " + methodName + " : " + "ERROR::WMIScripting object was not properly initialized.");
                CleanUpWMI();
            }
            else
            {
                //run immediately but continue other processing
                setTimeout(GetProcessorCounters, 1);
                
                 if (Number(_forceWMITimeoutID) > 0)
                    clearTimeout(_forceWMITimeoutID);
                //create new timer to check for hung WMI request
                _forceWMITimeoutID = setTimeout(ForceWMITimeout, POLLING_TIMEOUT + 5000);
            }
        }
        else if(_settingsWindowOpen == true)
            debugLog(new Date() + ":  " + "settings window is open ... skipping"); 
        else if(_processingWMI == true)
            debugLog(new Date() + ":  " + "still processing... skipping"); 
    
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception executing WMI queries." + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}

/*
    InitWMI - initializes WMI Scripting objects
*/
function InitWMI()
{
    var methodName = "InitWMI";
    var init = false;
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
//        debugger;
        
        _locator = new ActiveXObject ("WbemScripting.SWbemLocator");
        if(_local)
        {
            _service = _locator.ConnectServer(".","root\\cimv2");
            init  = true;
        }
        else
        {
            if (_computer == "" || _domain == "" || _username == "")
            {
                debugLog(new Date() + ":  " + methodName + " : " + "ERROR::Connection information has not been configured properly.");
                tooltipError = "Connection information has not been configured properly.";
                DisplayWMIError(tooltipError);
                return false;
            }
            try
            {
                _service = _locator.ConnectServer(_computer, 
                                                "root\\cimv2", 
                                                _username, 
                                                _password, 
                                                "MS_409", 
                                                "ntlmdomain:" + _domain);
                init = true;
                
            } catch (ex) {
                var message = ex.message;
                var tooltipError = "";
                //display connection error in tooltips
                if (message.match("Access is denied"))
                    tooltipError = "Invalid username/password for remote computer" + "\n" + "Access is denied";
                else if(message.match("RPC"))
                    tooltipError = "Remote computer is unavailable" + "\n" + message;
                else
                    tooltipError = "Unable to connect to remote machine to retrieve WMI counters" + "\n" + message;
                //update tooltips for data fields
                DisplayWMIError(tooltipError);
                CleanUpWMI();
                
                init = false;
            }
        }
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception creating WMI scripting object" + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
        CleanUpWMI();
        init = false;
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
        return init;
    }
}

/*
    CleanUpWMI - 'dispose' _locator object and _service object
*/
function CleanUpWMI()
{
    var methodName = "CleanUpWMI";
    
    try
    {
//        debugger;
        debugLog(new Date() + ":  " + methodName + " start");
        
        if(_locator != null)
            _locator = null;
        if(_service != null)
            _service = null;
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception configuring WMI scripting object" + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        _processingWMI = false;
        debugLog(new Date() + ":  " + methodName + " end");
    }
}

/*
    GetProcessorCounters    -   query first set of processor counters for calculation of CPU usage
        - Processor.MaxClockSpeed
        - PercentProcessorTime._Total
        - PercentIdleTime
*/
function GetProcessorCounters()
{
//    debugger;
    var methodName = "GetProcessorCounters";
    var success = false;
    var procTime = -1;
    var timestamp = -1;
    var procTimes = new Array();
    var timestamps = new Array();
    
    try
    {        
        //usage statistics
        var procs = _service.ExecQuery("SELECT * FROM Win32_PerfRawData_PerfOS_Processor WHERE Name IS NOT NULL");
        var e = new Enumerator (procs);
        for (;!e.atEnd();e.moveNext ())
        {
            var p = e.item();
            if(p.Name.toLowerCase() == "_total")
            {
                procTime = p.PercentProcessorTime;
                timestamp = p.TimeStamp_Sys100NS;
            }
            else
            {
                procTimes.push(p.PercentProcessorTime);
                timestamps.push(p.TimeStamp_Sys100NS);
            }
        }
        _procTime1 = procTime;
        _procTimes1 = procTimes;
        _timestamp1 = timestamp;
        _timestamps1 = timestamps;
    
        success = true;
        
    } catch(ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception retrieving Processor WMI Counters " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
        _procTime1 = -1;
        _procTimes1 = new Array();
        _timestamp1 = -1;
        _timestamps1 = new Array();
        success = false;
    } finally {
        //wait before querying 2nd time
        setTimeout(GetProcessorCounters2, 1000);
    }
}
/*
    GetProcessorCounters2 - query second set of processor counters for calculation of CPU usage
*/
function GetProcessorCounters2()
{
//    debugger;
    var methodName = "GetProcessorCounters2";
    var success = false;
    var procCount = 0;
    var procSpeed = -1;
    var procTime = -1;
    var procTimes = new Array();
    var procTime2 = -1;
    var timestamp2 = -1;
    var procTimes2 = new Array();
    var timestamps2 = new Array();
    
    try
    {
        //retrieve processor details
        var procDetails = _service.ExecQuery("SELECT * FROM Win32_Processor WHERE Name IS NOT NULL");
        
        var e = new Enumerator (procDetails);
        for (;!e.atEnd();e.moveNext ())
        {
            var p = e.item();
            
            var speed = Number(p.MaxClockSpeed) / 1000;
            procSpeed = Math.round(speed * 10) / 10;
            break;  //same speed for all CPUs
        }
        
        if(_procTime1 < 0)
        {
            _procTime = -1;
            _procTimes = new Array();
        }
        else
        {
            //usage statistics
            var procs = _service.ExecQuery("SELECT * FROM Win32_PerfRawData_PerfOS_Processor WHERE Name IS NOT NULL");
            var e = new Enumerator (procs);
            for (;!e.atEnd();e.moveNext ())
            {
                var p = e.item();
                if(p.Name.toLowerCase() == "_total")
                {
                    procTime2 = p.PercentProcessorTime;
                    timestamp2 = p.TimeStamp_Sys100NS;
                }
                else
                {
                    procTimes2.push(p.PercentProcessorTime);
                    timestamps2.push(p.TimeStamp_Sys100NS);
                }
            }
            
            //calculate processor times
            procTime = (1 - ((procTime2 - _procTime1)/(timestamp2 - _timestamp1)))*100;
            procTime = Math.round(procTime);
            
            var p = 0;
            for (p; p < _procTimes1.length; p++)
            {
                procCount += 1;
                var time = (1 - ((procTimes2[p] - _procTimes1[p])/(timestamps2[p] - _timestamps1[p])))*100;
                time = Math.round(time);
                if(time > 100)
                    time = -1;
                procTimes.push(time);
            }
            
            if(procCount < 1)
                procCount = 1;
            if(_procCount != procCount)
            {
                _procCount = procCount;
                //run immediately but continue other processing
                setTimeout('UpdatePositions(\'' + _procCount + '\',' + _open + ')', 1);
            }
                
            _procTime = procTime;
            _procTimes = procTimes;
            _procCount = procCount;
            _procSpeed = procSpeed;
        }
        
        success  = true;

    } catch(ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception retrieving Processor WMI Counters " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
        _procTime = -1;
        _procTimes = new Array();
        _procCount = -1;
        _procSpeed = -1;
        DisplayProcessorError(ex.message);
        success = false;
    } finally {
        //run immediately but continue other processing
        setTimeout(UpdateProcessorDisplay, 1);
        GetMemoryCounters();
    }
}

/*
    GetMemoryCounters   - PhysicalMemory.Capacity and Memory.AvailableMBytes
*/
function GetMemoryCounters()
{
//    debugger;
    var methodName = "GetMemoryCounters";
    var success = false;
    var totalBytes = -1;
    var totalMBytes = -1;
    var megaBytes = -1;
    var percentAvailable = -1;
    var percentUsed = -1;
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        //get total memory (bytes)
        var memory = _service.ExecQuery("SELECT * FROM Win32_PhysicalMemory");
        
        var e = new Enumerator (memory);
        for (;!e.atEnd();e.moveNext ())
        {
            var p = e.item();
            totalBytes += Number(p.Capacity);
        }
        totalMBytes = totalBytes / (1024 * 1024);
        totalMBytes = Math.round(totalMBytes * 10) / 10;
        
        //get available memory (Mbytes)
        var properties = _service.ExecQuery("Select * From Win32_PerfRawData_PerfOS_Memory");
        
        var e = new Enumerator (properties);
        for (;!e.atEnd();e.moveNext ())
        {
            var p = e.item();
            megaBytes = Number(p.AvailableMBytes);
        }
        
        if(totalBytes > 0)
        {
            percentAvailable = Math.round( (100 * (megaBytes) / totalMBytes) );
            percentUsed = 100.0 - percentAvailable;
        }
        else
            percentUsed = -1;
                
        success = true;
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception retrieving Memory WMI Counters " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
        percentUsed = -1;
        totalMBytes = -1;
        DisplayMemoryError(ex.message);
        success = false;
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
        //run immediately but continue other processing
        setTimeout('UpdateMemoryDisplay(\'' + percentUsed + '\',' + totalMBytes + ')', 1);
        GetSystemCounters();
        return success;
    }
}

/*
    GetSystemCounters
        - Processes
        - Threads
*/
function GetSystemCounters()
{
    var methodName = "GetSystemCounters";
    var processes = 0;
    var threads = 0;
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        var properties = _service.ExecQuery("Select * From Win32_PerfRawData_PerfOS_System");
        
        var e = new Enumerator (properties);
        for (;!e.atEnd();e.moveNext ())
        {
            var p = e.item();
            processes = Number(p.Processes);
            threads = Number(p.Threads);
        }
        
        if(processes < 0)
        {
            processCount.style.color = "Red";
            processCount.title = "Unable to retrieve Active System Processes count";
            processCount.innerText = "error";
        }
        else
        {
            processCount.style.color = "White";
            processCount.title = "# Active System Processes";
            processCount.innerText = processes.toString();
        }
        
        if(threads < 0)
        {
            systemThreadCount.style.color = "Red";
            systemThreadCount.title = "Unable to retrieve Active System threads count";
            systemThreadCount.innerText = "error";
        }
        else
        {
            systemThreadCount.style.color = "White";
            systemThreadCount.title = "# Active System Threads";
            systemThreadCount.innerText = threads.toString();
        }
        
        success = true;
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception retrieving System WMI counters " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
        DisplaySystemError(ex.message);
        success = false;
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
        CleanUpWMI();
        _processingWMI = false;
        return success;
    }
}

/*
    ForceWMITimeout - //this is a fail safe to keep gadget from waiting for 'lost' request
*/
function ForceWMITimeout()
{
    var methodName = "ForceWMITimeout";
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        if (_processingWMI == true)
        {
            var queryExecutionEnd = new Date();
            var diffTime = queryExecutionEnd.getTime() - _queryExecutionStart.getTime();    //ms
            debugLog(new Date() + ":  **** forceWMITimeout processingWMI is true " + diffTime);
            if (Number(diffTime) >= POLLING_TIMEOUT)  //if the time difference is greater than timeout setting
            {
                debugLog(new Date() + ":  **** forceHTTPTimeout - The timeout for WMI query execution was exceeded");
                CleanUpWMI();
                DisplayWMIError("WMI Queries Timed Out.");
                _processingWMI = false; //don't wait for last query to return
            }
        }
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}

/**************************************************************************************************
    END WMI Query execution
**************************************************************************************************/




/**************************************************************************************************
    Display updates
**************************************************************************************************/

/*
    ClearDisplay - clear current values from all display fields
*/
function ClearDisplay()
{
    var methodName = "ClearDisplay";
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");

        //total processor counter values
        processorTimeInner.style.height = "0%";
        processorTime.style.color = "White";
        processorTime.innerText = "...";
        processorTime.title = "Processing...";
        processorStatistics.style.color = "White";
        processorStatistics.innerText = "...";
        processorStatistics.title = "Processing...";
        //individual processor counter values
        proc0inner.style.width = "0%";
        proc0.style.color = "White";
        proc0.innerText = "...";
        proc0.title = "";
        proc1inner.style.width = "0%";
        proc1.style.color = "White";
        proc1.innerText = "...";
        proc1.title = "";
        proc2inner.style.width = "0%";
        proc2.style.color = "White";
        proc2.innerText = "...";
        proc2.title = "";
        proc3inner.style.width = "0%";
        proc3.style.color = "White";
        proc3.innerText = "...";
        proc3.title = "";
        //memory counter values
        percentMemoryUsedInner.style.height = "0%";
        percentMemoryUsed.style.color = "White";
        percentMemoryUsed.innerText = "...";
        percentMemoryUsed.title = "Processing...";
        totalMemoryMB.style.color = "White";
        totalMemoryMB.innerText = "...";
        totalMemoryMB.title = "Processing...";
        //system counter values
        processCount.style.color = "White";
        processCount.innerText = "...";
        processCount.title = "Processing...";
        systemThreadCount.style.color = "White";
        systemThreadCount.innerText = "...";
        systemThreadCount.title = "Processing...";
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}

/*
    DisplayWMIError - show WMI error in all text field tooltips
*/
function DisplayWMIError(errorText)
{
    var methodName = "DisplayWMIError";
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        //total processor counter values
        processorTimeInner.style.height = "0%";
        processorTime.style.color = "Red";
        processorTime.innerText = "error";
        processorTime.title = errorText;
        processorStatistics.style.color = "Red";
        processorStatistics.innerText = "error";
        processorStatistics.title = errorText;
        //individual processor counter values
        proc0inner.style.width = "0%";
        proc0.style.color = "Red";
        proc0.innerText = "error";
        proc0.title = errorText;
        proc1inner.style.width = "0%";
        proc1.style.color = "Red";
        proc1.innerText = "error";
        proc1.title = errorText;
        proc2inner.style.width = "0%";
        proc2.style.color = "Red";
        proc2.innerText = "error";
        proc2.title = errorText;
        proc3inner.style.width = "0%";
        proc3.style.color = "Red";
        proc3.innerText = "error";
        proc3.title = errorText;
        //memory counter values
        percentMemoryUsedInner.style.height = "0%";
        percentMemoryUsed.style.color = "Red";
        percentMemoryUsed.innerText = "error";
        percentMemoryUsed.title = errorText;
        totalMemoryMB.style.color = "Red";
        totalMemoryMB.innerText = "error";
        totalMemoryMB.title = errorText;
        //system counter values
        processCount.style.color = "Red";
        processCount.innerText = "error";
        processCount.title = errorText;
        systemThreadCount.style.color = "Red";
        systemThreadCount.innerText = "error";
        systemThreadCount.title = errorText;
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
        CleanUpWMI();
        _processingWMI = false;
    }
}
function DisplayProcessorError(errorText)
{
    //total processor counter values
    processorTimeInner.style.height = "0%";
    processorTime.style.color = "Red";
    processorTime.innerText = "error";
    processorTime.title = errorText;
    processorStatistics.style.color = "Red";
    processorStatistics.innerText = "error";
    processorStatistics.title = errorText;
    //individual processor counter values
    proc0inner.style.width = "0%";
    proc0.style.color = "Red";
    proc0.innerText = "error";
    proc0.title = errorText;
    proc1inner.style.width = "0%";
    proc1.style.color = "Red";
    proc1.innerText = "error";
    proc1.title = errorText;
    proc2inner.style.width = "0%";
    proc2.style.color = "Red";
    proc2.innerText = "error";
    proc2.title = errorText;
    proc3inner.style.width = "0%";
    proc3.style.color = "Red";
    proc3.innerText = "error";
    proc3.title = errorText;
}
function DisplayMemoryError(errorText)
{
    //memory counter values
    percentMemoryUsedInner.style.height = "0%";
    percentMemoryUsed.style.color = "Red";
    percentMemoryUsed.innerText = "error";
    percentMemoryUsed.title = errorText;
    totalMemoryMB.style.color = "Red";
    totalMemoryMB.innerText = "error";
    totalMemoryMB.title = errorText;
}
function DisplaySystemError(errorText)
{
    //system counter values
    processCount.style.color = "Red";
    processCount.innerText = "error";
    processCount.title = errorText;
    systemThreadCount.style.color = "Red";
    systemThreadCount.innerText = "error";
    systemThreadCount.title = errorText;
}

/*
    OpenCloseSlider - expand/contract dVUE to show/hide individual CPU values
*/
function OpenCloseSlider()
{
    var methodName = "OpenCloseSlider";
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        UpdatePositions(_procCount, !_open);
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}

/*
    UpdatePositions - (re)position all divs based on number of processors and slider state
        - procCount     number of processors
        - setToOpen     true = open / false = close
*/
function UpdatePositions(procCount, setToOpen)
{
//    debugger;
    var methodName = "UpdatePositions";
    var procCountDiv;
    var threadCountDiv;
    var hostnameDiv;
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        procCountDiv = document.getElementById("processCount");
        threadCountDiv = document.getElementById("systemThreadCount");
        hostnameDiv = document.getElementById("hostname");
        
        if(procCount < 1)
            procCount = 1;
        if(procCount == 1)
        {//use all default position settings and basic background image(no open state allowed)
            System.Gadget.background = CLOSED_SINGLE_BACKGROUND;
            document.body.style.height = CLOSED_BODY_HEIGHT;
            procCountDiv.style.top = CLOSED_PROCESSCOUNT_TOP;
            threadCountDiv.style.top = CLOSED_THREADCOUNT_TOP;
            hostnameDiv.style.top = CLOSED_HOSTNAME_TOP;
            //hide all individual cpu bars and fields
            proc0inner.style.visibility = "hidden";
            proc0.style.visibility = "hidden";
            proc1inner.style.visibility = "hidden";
            proc1.style.visibility = "hidden";
            proc2inner.style.visibility = "hidden";
            proc2.style.visibility = "hidden";
            proc3inner.style.visibility = "hidden";
            proc3.style.visibility = "hidden";
            //disable slider button
            sliderButton.title = "";
            sliderButton.style.cursor = "default";
            //set sliderstatus variable to closed
            _open = false;
        }
        else if(procCount > 1 && setToOpen == false)
        {//use default position settings but switch background image (with arrow)
            System.Gadget.background = CLOSED_MULTIPLE_BACKGROUND;
            document.body.style.height = CLOSED_BODY_HEIGHT;
            procCountDiv.style.top = CLOSED_PROCESSCOUNT_TOP;
            threadCountDiv.style.top = CLOSED_THREADCOUNT_TOP;
            hostnameDiv.style.top = CLOSED_HOSTNAME_TOP;
            //hide all individual cpu bars and fields
            proc0inner.style.visibility = "hidden";
            proc0.style.visibility = "hidden";
            proc1inner.style.visibility = "hidden";
            proc1.style.visibility = "hidden";
            proc2inner.style.visibility = "hidden";
            proc2.style.visibility = "hidden";
            proc3inner.style.visibility = "hidden";
            proc3.style.visibility = "hidden";
            //enable slider button
            sliderButton.title = "Click here to view individual CPUs display.";
            sliderButton.style.cursor = "hand";
            //set sliderstatus variable to closed
            _open = false;
        }
        else if(procCount > 1 && procCount < 4 && setToOpen == true)
        {//use position settings for 2 horizontal bars and switch to 2 cpu open background image
            System.Gadget.background = OPEN_2_BACKGROUND;
            document.body.style.height = OPEN_2_BODY_HEIGHT;
            procCountDiv.style.top = OPEN_2_PROCESSCOUNT_TOP;
            threadCountDiv.style.top = OPEN_2_THREADCOUNT_TOP;
            hostnameDiv.style.top = OPEN_2_HOSTNAME_TOP;
            //only show 2 individual cpu bars and fields
            proc0inner.style.visibility = "visible";
            proc0.style.visibility = "visible";
            proc1inner.style.visibility = "visible";
            proc1.style.visibility = "visible";
            proc2inner.style.visibility = "hidden";
            proc2.style.visibility = "hidden";
            proc3inner.style.visibility = "hidden";
            proc3.style.visibility = "hidden";
            //enable slider button
            sliderButton.title = "Click here to close individual CPUs display.";
            sliderButton.style.cursor = "hand";
            //set sliderstatus variable to open
            _open = true;
        }
        else if(procCount >= 4 && setToOpen == true)
        {//use position settings for 2 horizontal bars and switch to 4 cpu open background image
            System.Gadget.background = OPEN_4_BACKGROUND;
            document.body.style.height = OPEN_4_BODY_HEIGHT;
            procCountDiv.style.top = OPEN_4_PROCESSCOUNT_TOP;
            threadCountDiv.style.top = OPEN_4_THREADCOUNT_TOP;
            hostnameDiv.style.top = OPEN_4_HOSTNAME_TOP;
            //show all individual cpu bars and fields
            proc0inner.style.visibility = "visible";
            proc0.style.visibility = "visible";
            proc1inner.style.visibility = "visible";
            proc1.style.visibility = "visible";
            proc2inner.style.visibility = "visible";
            proc2.style.visibility = "visible";
            proc3inner.style.visibility = "visible";
            proc3.style.visibility = "visible";
            //enable slider button
            sliderButton.title = "Click here to close individual CPUs display.";
            sliderButton.style.cursor = "hand";
            //set sliderstatus variable to open
            _open = true;
        }
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}


/*
    UpdateProcessorDisplay - display % Processor Time _Total in main window 
        and individual processors' % Processor Time in slide down window
*/
function UpdateProcessorDisplay()
{
//    debugger;
    var methodName = "UpdateProcessorDisplay";
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        if(_procTime < 0 || _procTime > 100)
        {
            processorTime.style.color = "Red";
            processorTime.innerText = "error";
            processorTime.title = "Error retrieving % processor time.";
        }
        else
        {
            processorTime.style.color = "White";
            processorTime.innerText = _procTime.toString() + "%";
            processorTime.title = _procTime.toString() + "% Processor Utilization";
        }
        
        if (_procTimes != null && _procTimes.length > 0)
            _procCount = _procTimes.length;
        
        if (_procCount < 0 || _procSpeed < 0)
        {
            processorStatistics.style.color = "Red";
            processorStatistics.innerText = "error";
            processorStatistics.title = "Unable to retrieve processor details.";
        }
        else
        {
            processorStatistics.style.color = "White";
            processorStatistics.innerText = _procCount.toString() + " @ " + _procSpeed.toString() + " GHz";
            processorStatistics.title = _procCount.toString() + " logical CPUs with Max ClockSpeed of " + _procSpeed.toString() + " GHz";
        }
        
        UpdateProcessorDisplayGraphic(_procTime, _procTimes);
    
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        ResetProcessorSettings();
        debugLog(new Date() + ":  " + methodName + " end");
    }
}
/*
    UpdateProcessorDisplayGraphic - configure vertical bar height for total cpu 
        and horizontal bar width for individual cpus
        - totalPercent      - aggreagated % ProcessorTime for all processors
        - cpuPercentages    - array of % ProcessorTime values for each individual processor
*/
function UpdateProcessorDisplayGraphic(totalPercent, cpuPercentages)
{
    var methodName = "UpdateProcessorDisplayGraphic";
    var count = 0;
    
    try
    {
//        debugger;
        debugLog(new Date() + ":  " + methodName + " start");
        
        if(totalPercent <= 0)
        {
            processorTimeInner.innerHTML = "<img src='" + PROCESSOR_VERTICAL_TOP_IMAGE + "' style='position:absolute; top:0px; vertical-align:top; visibility:hidden;'>";
            processorTimeInner.style.height = "0%";
        }
        else
        {
            processorTimeInner.innerHTML = "<img src='" + PROCESSOR_VERTICAL_TOP_IMAGE + "' style='position:absolute; top:0px; vertical-align:top;'>";
            processorTimeInner.style.height = totalPercent + "%";
        }
        processorTimeInner.style.backgroundImage = "url(images/icons/green_bar.png)";
     
        //get number of CPUs
        if(cpuPercentages != null && cpuPercentages.length > 0)
            count = cpuPercentages.length;
        else
            count = 1;
            
        var i = 0;
        for (i; i<count; i++)
        {//get and display html for individual CPUs
            if (i > 3)// only supports diplaying up to 4 individual CPUs
                break;
                
            var percent = cpuPercentages[i];
            var bar = document.getElementById("proc" + i.toString() + "inner");
            var number = document.getElementById("proc" + i.toString());
            bar.style.backgroundImage = "url(images/icons/green_horizontal.png)";
            if(percent < 0)
            {
                bar.style.width = "0%";
                number.style.color = "Red";
                number.innerText = "error";
            }
            else
            {
                bar.style.width = percent + "%";
                number.style.color = "White";
                number.innerText = percent.toString() + "%";
            }
        }
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}
function ResetProcessorSettings()
{
    _procTime1 = -1;
    _timestamp1 = -1;
    _procTimes1 = new Array();
    _timestamps1 = new Array();

    _procTime = -1;
    _procTimes = new Array();
    _procSpeed = -1;
}

/*
    UpdateMemoryDisplay - configure vertical bar height for percent memory used 
        - percentUsed   - percent of memory in use
        - totalMBytes   - total physical memory
*/
function UpdateMemoryDisplay(percentUsed, totalMBytes)
{
    var methodName = "UpdateMemoryDisplay";
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        if (percentUsed < 0 || percentUsed > 100)
        {
            percentMemoryUsed.style.color = "Red";
            percentMemoryUsed.innerText = "error";
            percentMemoryUsed.title = "Unable to retrieve Physical Memory counters.";
        }
        else
        {
            percentMemoryUsed.style.color = "White";
            percentMemoryUsed.innerText = percentUsed.toString() + "%";
            percentMemoryUsed.title = percentUsed.toString() + "% of total physical memory in use";
        }
        
        UpdateMemoryDisplayGraphic(percentUsed);
            
        if (totalMBytes < 0)
        {
            totalMemoryMB.style.color = "Red";
            totalMemoryMB.innerText = "error";
            totalMemoryMB.title = "Unable to retrieve Physical Memory details.";
        }
        else
        {
            totalMemoryMB.style.color = "White";
            totalMemoryMB.innerText = "of " + totalMBytes.toString() + " MB";
            totalMemoryMB.title = totalMBytes.toString() + " MB of total Physical Memory";
        }
        
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}
/*
    UpdateMemoryDisplayGraphic - configure vertical bar height for total memory used
        - percentUsed      - aggreagated % ProcessorTime for all processors
*/
function UpdateMemoryDisplayGraphic(percentUsed)
{
    var methodName = "UpdateMemoryDisplayGraphic";
    var html = "";
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");

        if(percentUsed <= 0)
        {
            percentMemoryUsedInner.innerHTML = "<img src='" + MEMORY_TOP_IMAGE + "' style='position:absolute; top:0px; vertical-align:top; visibility:hidden;'>";
            percentMemoryUsedInner.style.height = "0%";
        }
        else
        {
            percentMemoryUsedInner.innerHTML = "<img src='" + MEMORY_TOP_IMAGE + "' style='position:absolute; top:0px; vertical-align:top;'>";
            percentMemoryUsedInner.style.height = percentUsed + "%";
        }
        percentMemoryUsedInner.style.backgroundImage = "url(images/icons/blue_bar.png)";
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}

/**************************************************************************************************
    END Display updates
**************************************************************************************************/


/*
    MethodTemplate - method description
*/
function MethodTemplate()
{
    var methodName = "MethodTemplate";
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}