/************************************************************************/
/* Copyright 2004-2007 SYS Technologies, Inc.  All rights reserved.     */
/*               SYS Technologies, Inc. - Proprietary                   */
/* This software contains information of SYS Technologies and is not    */
/* to be disclosed or used except in accordance with applicable         */
/* agreements. Use of this DOPPLER VUE software is governed as          */
/* expressly authorized in the relevant agreement between               */
/* SYS Technologies and the customer, as well as copyright laws and     */
/* international treaties. Unauthorized distribution or duplication     */
/* will result in severe criminal penalties.                            */
/*               SYS Technologies, Inc. - Proprietary                   */
/************************************************************************/

/**************************************************************************************************
    GLOBAL MEMBERS
**************************************************************************************************/

var _interval = 5000            // configured polling interval
var _intervalsList;             // array of allowed polling intervals

var _local = true;
var _computer = "";
var _domain = "";
var _username = "";
var _password = "";
var _validate = true;

var DEFAULT_INTERVAL = 5000;    // ms - 5 seconds

/**************************************************************************************************
    begin logging
**************************************************************************************************/
var oFSO = new ActiveXObject("Scripting.FileSystemObject");
var bDebug = oFSO.FileExists(System.Gadget.path + "\\debugsettings.txt");
try{
	if (bDebug)
		var debugLogFile = oFSO.OpenTextFile(System.Gadget.path + "\\debugsettings.txt", 8);
} catch(err) {bDebug = false; debugLog("Open debugsettings.txt error"+err.name+" - "+err.message)}

/*
    debugLog - debug log utility
*/
function debugLog(str) {
	try{
		System.Debug.outputString(str);
		if (bDebug)
			debugLogFile.WriteLine(str);
	} catch(err) {}
}
/**************************************************************************************************
    end logging
**************************************************************************************************/

/**************************************************************************************************
    END GLOBAL MEMBERS
**************************************************************************************************/


function loadSettings()
{
    //debugger;
	debugLog(new Date() + ":  " + "loadSettings start");
    // init event methods
    System.Gadget.onSettingsClosing = procSettingsClosingEvent;                 // set settings page closing event
    
    // get settings data
    var tempInterval = System.Gadget.Settings.read( "pollingInterval" );
    if(tempInterval == "")
        _interval = DEFAULT_INTERVAL;
    else
        _interval = tempInterval;
    
    // add data to Settings page
    loadIntervalsList();
    populateIntervalSelection();
    populateAuthenticationSelections();
    
    debugLog(new Date() + ":  " + "loadSettings end");
}

function loadIntervalsList()
{
    _intervalsList = new Array();
    _intervalsList.push(5000);
    _intervalsList.push(10000);
    _intervalsList.push(15000);
    _intervalsList.push(20000);
    _intervalsList.push(25000);
    _intervalsList.push(30000);
}
function populateIntervalSelection()
{
    var selectedIndex = 0;
    if(_intervalsList.length > 0)
    {
        var i = 0;
        for (i; i < _intervalsList.length; i++)
        {
            if(_intervalsList[i] > 0)
            {
                pollingInterval.options[i] = new Option( _intervalsList[i]/1000 + " seconds", _intervalsList[i] );
                if(_intervalsList[i] == _interval)
                    selectedIndex = i;
            }
        }
    }
    
    pollingInterval.options.selectedIndex = selectedIndex;
}
function populateAuthenticationSelections()
{
    var tempLocal = System.Gadget.Settings.read( "local" );
    if(tempLocal.length == 0)
        _local = true;
    else
        _local = tempLocal;
        
    _computer = System.Gadget.Settings.read( "computer" );
    _domain = System.Gadget.Settings.read( "domain" );
    _username = System.Gadget.Settings.read( "username" );
    _password = System.Gadget.Settings.read( "password" );
    
    computer.value = _computer;
    domain.value = _domain;
    username.value = _username;
    password.value = _password;
    
    changeLocalRemote(_local);
}

/*
    procSettingsClosingEvent - process settings page closing event
*/
function procSettingsClosingEvent(event)
{	// process settings page closing event
    debugLog(new Date() + ":  " + "procSettingsClosingEvent start");
    //debugger;
    //save settings
    //polling interval
    _interval = pollingInterval.options[ pollingInterval.options.selectedIndex ].value;
    System.Gadget.Settings.write( "pollingInterval", _interval );
    
    //_local is set on page load and when radio button is selected
    
    //remote machine authentication
    _computer = computer.value;
    _domain = domain.value;
    _username = username.value;
    _password = password.value;
    System.Gadget.Settings.write( "local", _local );
    System.Gadget.Settings.write( "computer", _computer );
    System.Gadget.Settings.write( "domain", _domain );
    System.Gadget.Settings.write( "username", _username );
    System.Gadget.Settings.write( "password", _password );

    if (event != null && event.closeAction == event.Action.commit)
    {//validate connection settings
        var validateWMI = true;

        //validate remote connection settings before closing
            //prevents gadget from trying to connect every interval with invalid settings
            //repeated connections with bad password will eventually lock the user out from WMI
//        if(!_local && _validate == true)
//            validateWMI = ValidateConnectionSettings(_computer, _domain, _username, _password);
//            
//        if (!validateWMI)
//        {
//            event.cancel = true;
//            return;
//        }
        //validation is only performed on the 'Connection' button click
        
    }
    event.cancel = false;
    
    debugLog(new Date() + ":  " + "procSettingsClosingEvent end");
}


function local_onclick()
{
    changeLocalRemote(true);
}
function remote_onclick()
{
    changeLocalRemote(false);
}

function changeLocalRemote(isLocal)
{
    var methodName = "changeLocalRemote";
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
        
        //clear connectionError and connectionMessage labels
        connectionError.innerText = " ";
        connectionMessage.innerText = " ";
        
        if(isLocal)
        {//disable all remote settings controls
            _local = true;
            radioLocal.checked = true;
            radioRemote.checked = false;
            authentication.disabled = true;
            computer.disabled = true;
            domain.disabled = true;
            username.disabled = true;
            password.disabled = true;
            validate.disabled = true;
        }
        else
        {//enable all remote settings controls
            _local = false;
            radioLocal.checked = false;
            radioRemote.checked = true;
            authentication.disabled = false;
            computer.disabled = false;
            domain.disabled = false;
            username.disabled = false;
            password.disabled = false;
            connectionError.disabled = false;
            connectionMessage.disabled = false;
            validate.disabled = false;
        }
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}

function TestConnection()
{
    //read form values and assign to class variables
    _computer = computer.value;
    _domain = domain.value;
    _username = username.value;
    _password = password.value;
        
    PrepareForConnectionTest();
    window.setTimeout(ValidateConnectionSettings, 500);
}
/*
    PrepareForConnectionTest - clear error and message fields for connection test
*/
function PrepareForConnectionTest()
{
    var methodName = "PrepareForConnectionTest";
    
    try
    {
        //debugLog(new Date() + ":  " + methodName + " start");
        
        connectionError.style.color = "Black";
        connectionError.innerText = "Attempting to connect...";
        connectionMessage.innerText = " ";
        
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
    }
}
/*
    ValidateConnectionSettings - check WMI Connection before closing window
        invalid connection setting will eventually lock the specified user out of WMI
*/
function ValidateConnectionSettings()
{
//    debugger;
    var methodName = "ValidateConnectionSettings";
    var validated = true;
    var locator = null;
    var service = null;
    
    try
    {
        debugLog(new Date() + ":  " + methodName + " start");
                
        //test connection
        if(_local)
            validated = true;
        else
        {
            try
            {
                if(_computer == "" && _domain == "" && _username == "" && _password == "")
                {
                    validated = false;
                    window.setTimeout('DisplayConnectionTestResults(true, "Unable to connect.", "No authentication settings specified.")', 1);
                }
                else
                {
                    locator = new ActiveXObject ("WbemScripting.SWbemLocator");
                    service = locator.ConnectServer(_computer, 
                                                    "root\\cimv2", 
                                                    _username, 
                                                    _password, 
                                                    "MS_409", 
                                                    "ntlmdomain:" + _domain);

                    window.setTimeout('DisplayConnectionTestResults(false, "Connection successful.", " ")', 1);
                    validated = true;
                }
                
            } catch (ex) {
                var message = ex.message;
                validated = false;
                
                if (message.match("Access is denied"))  //display authentication failure message to user
                    window.setTimeout('DisplayConnectionTestResults(true, "Access is denied.", "Click OK to continue saving settings.")', 1);
                else if (message.match("RPC server is unavailable"))    //display server unavailable message to user
                    window.setTimeout('DisplayConnectionTestResults(true, "Server is Unavailable.", "Click OK to continue saving settings.")', 1);
                else if (message.match('User credentials cannot be used for local connections'))
                    window.setTimeout('DisplayConnectionTestResults(true, "User credentials cannot be used for local connections.", "Click OK to continue saving settings.")', 1);
                else    //display 'other' failure message to user
                    window.setTimeout('DisplayConnectionTestResults(true, "Unknown connection failure.", "Click OK to continue saving settings.")', 1);
                    
            }
        }
            
    } catch (ex) {
        var catchMsg = new Date() + ":  " + methodName + " : " + "ERROR::Exception... " + 
            "\n\t" + "name = " + ex.name + 
            "\n\t" + "message = " + ex.message;
        debugLog(catchMsg);
        connectionError.style.color = "Red";
        connectionError.innerText = ex.message;
        validated = false;
    } finally {
        debugLog(new Date() + ":  " + methodName + " end");
        //clean up WMI objects
        if(locator != null)
            locator = null;
        if(service != null)
            service = null;
            
        return validated;
    }
}
/*
    DisplayConnectionTestResults
*/
function DisplayConnectionTestResults(isError, statusText, messageText)
{
//    debugger;
    
    if(isError)
        connectionError.style.color = "Red";
    else
        connectionError.style.color = "Black";
    
    connectionError.innerText = statusText;
    connectionMessage.innerText = messageText;
}